/* メモリ定義
    チップと用途に応じてメモリ領域を変更のこと
    - MEMORYセクション
    - _RAM_SIZE
    の2つを書き換える必要がある
    FLASH領域の定義に際してはページサイズ（1 kbytes or 2kbytes）を考慮すること

  RAM        static用RAM領域 + main用RAM領域 + ヒープ領域 + スタック領域
  FLASH      mainプログラム用FLASH領域
  FLASHst    staticプログラム用FLASH領域
  FLOG       FLASHへのログデータ用FLASH領域
  MAZE       迷路データ保存用FLASH領域
  PARAMETER  パラメータ保存用FLASH領域
*/

MEMORY
{
  RAM (rwx)    : ORIGIN = 0x20000000, LENGTH = 64k
  VECTOR (rx)  : ORIGIN = 0x08000000, LENGTH = 16k
  FLASH (rx)   : ORIGIN = 0x08004000, LENGTH = 112k
  FLASHst (rx) : ORIGIN = 0x08020000, LENGTH = 384k
  FLOG (r)     : ORIGIN = 0x08060000, LENGTH = 0x0019000 /* 100k */
  MAZE (r)     : ORIGIN = 0x08079000, LENGTH = 8k        /* +8k for spare */
  PARAMETER (r): ORIGIN = 0x0807d000, LENGTH = 4k        /* +4k for spare */
}

/* ヒープとスタック用に最低限確保しておく容量 */
/* この容量を下回るとリンクエラーとなる */
_Minimum_Stack_Size = 6k;

/* スタック終端位置 = RAM終端位置*/
/* スタートアップルーチン内でスタックポインタにこの値がセットさせる */
_RAM_SIZE = 64k;
_estack = 0x20000000 + _RAM_SIZE;

/* 下記変数の値はソース中で
   extern unsigned int _FLASH_LOG_START;
   unsigned int address = (unsigned int)&_FLASH_LOG_START;
   で利用できる．
*/
_FLASH_LOG_START  = ORIGIN(FLOG);
_FLASH_LOG_SIZE   = LENGTH(FLOG);
_FLASH_MAZE_START = ORIGIN(MAZE);
_FLASH_MAZE_SIZE  = LENGTH(MAZE);
_FLASH_PARM_START = ORIGIN(PARAMETER);
_FLASH_PARM_SIZE  = LENGTH(PARAMETER);
